package auth

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"net/http"
	"net/url"
	"path"

	lfcc "git.loafle.net/commons_go/config"
	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_probes/central/client"
	"git.loafle.net/overflow/overflow_probes/commons"
	"git.loafle.net/overflow/overflow_probes/config"
)

const (
	noAuthConfigFileName     = "noauth.json"
	noAuthHeaderNoAuthID     = "overFlow-NoAuth-ID"
	noAuthHeaderNoAuthRegist = "overFlow-NoAuth-Regist"
	noAuthHeaderSetNoAuthID  = "overFlow-Set-NoAuth-ID"
)

type AuthHandler interface {
	commons.Handler
}

type authHandlers struct {
	configDir        string
	noAuthConfigPath string
	noAuthConfig     config.NoAuthProbeConfig
	entryURL         string
	c                client.Client
}

type NoAuthProbe struct {
	APIKey      string            `json:"apiKey"`
	Description NoAuthDescription `json:"description"`
}

type NoAuthDescription struct {
	Host    *Host    `json:"host"`
	Network *Network `json:"network"`
}

func New(configDir string) AuthHandler {
	h := &authHandlers{
		configDir: configDir,
	}

	var err error
	var centralURL *url.URL
	var noAuthEntryPoint string
	var ok bool
	if centralURL, err = url.Parse(config.Config.Central.URL); nil != err {
		logging.Logger.Error(fmt.Sprintf("Auth: Central URL[%s] is not valid error[%v]", config.Config.Central.URL, err))
		return nil
	}

	if noAuthEntryPoint, ok = config.Config.Central.EntryPoints["noauth"]; !ok {
		logging.Logger.Error("Auth: NoAuth entry point of Central is not exist")
		return nil
	}

	centralURL.Path = path.Join(centralURL.Path, noAuthEntryPoint)
	h.entryURL = centralURL.String()

	h.noAuthConfigPath = path.Join(configDir, noAuthConfigFileName)

	conf := lfcc.New()
	if lfcc.Exists(h.noAuthConfigPath) {
		if err = conf.Load(&h.noAuthConfig, h.noAuthConfigPath); nil != err {
			logging.Logger.Error(fmt.Sprintf("Auth: Loading of NoAuth config file[%s] failed error[%v]", h.noAuthConfigPath, err))
		}
	} else {
		if err = lfcc.Save(h.noAuthConfig, h.noAuthConfigPath, true); nil != err {
			logging.Logger.Error(fmt.Sprintf("Auth: Saving of NoAuth config file[%s] failed error[%v]", h.noAuthConfigPath, err))
		}
	}

	return h
}

func (h *authHandlers) Start() error {
	var err error
	isRegist := true

	h.c = client.New()
	header := http.Header{}

	if "" != h.noAuthConfig.TempID {
		isRegist = false
		header[noAuthHeaderNoAuthID] = []string{h.noAuthConfig.TempID}
	} else {
		var enc string
		if enc, err = h.getRegistHeader(); nil != err {
			return err
		}
		header[noAuthHeaderNoAuthRegist] = []string{enc}
	}
	var res *http.Response
	if res, err = h.c.Dial(h.entryURL, header, 4096, 4096); nil != err {
		return err
	}

	if isRegist {
		noAuthID := res.Header.Get(noAuthHeaderSetNoAuthID)
		logging.Logger.Debug(fmt.Sprintf("Auth: NoAuthID: %s", noAuthID))
	}

	return nil
}

func (h *authHandlers) getRegistHeader() (string, error) {
	var err error
	nap := NoAuthProbe{
		APIKey: config.Config.Domain.APIKey,
	}
	nad := NoAuthDescription{}
	nap.Description = nad

	if nad.Host, err = getHost(); nil != err {
		return "", err
	}

	if nad.Network, err = getNetwork(); nil != err {
		return "", err
	}

	var buf []byte
	if buf, err = json.Marshal(nap); nil != err {
		return "", err
	}

	enc := base64.StdEncoding.EncodeToString(buf)

	return enc, nil
}

func (h *authHandlers) listen() {
	// 1. regist
	// 2. wait for accept auth
}

func (h *authHandlers) Shutdown(ctx context.Context) error {

	return nil
}
